@@ -28,9 +28,9 @@ void set2string(const std::set<std::string>& in, std::string* out,
   for (iter = in.begin(); iter != in.end(); ++iter) {
     if (!out->empty()) {
       out->push_back(sep);
-    };
+    }
     out->append(*iter);
-  };
+  }
 }
 
 /**
@@ -41,6 +41,24 @@ void set2string(const std::set<std::string>& in, std::string* out,
 void addLocationPerGene(
     const std::string& gene, const std::string& range, const std::string& fn,
     OrderedMap<std::string, std::map<std::string, int> >* location) {
+  // check required column headers
+  PvalFileFormat pvalHeader;
+  if (pvalHeader.open(fn) < 0) {
+    REprintf("File [ %s ] does not have valid file header \n", fn.c_str());
+    return;
+  }
+  const int PVAL_FILE_CHROM_COL = pvalHeader.get("CHROM");
+  const int PVAL_FILE_POS_COL = pvalHeader.get("POS");
+  const int PVAL_FILE_REF_COL = pvalHeader.get("REF");
+  const int PVAL_FILE_ALT_COL = pvalHeader.get("ALT");
+
+  if (PVAL_FILE_CHROM_COL < 0 || PVAL_FILE_POS_COL < 0 ||
+      PVAL_FILE_REF_COL < 0 || PVAL_FILE_ALT_COL < 0) {
+    REprintf("Study [ %s ] does not have all required headers.\n", fn.c_str());
+    return;
+  }
+
+  // read each p-val file
   TabixReader tr(fn);
   tr.addRange(range);
   tr.mergeRange();
@@ -48,12 +66,25 @@ void addLocationPerGene(
   std::vector<std::string> fd;
   std::string key;
   int val;
+  std::map<std::string, int> keySeen;
   while (tr.readLine(&line)) {
     stringNaturalTokenize(line, "\t ", &fd);
     if (fd.size() < 2) continue;
-    key = fd[0] + ":" + fd[1];
-    if ((*location)[gene].count(key))  // not update existing variant
+    key = fd[PVAL_FILE_CHROM_COL] + ":" + fd[PVAL_FILE_POS_COL] + "_" +
+          fd[PVAL_FILE_REF_COL] + "/" + fd[PVAL_FILE_ALT_COL];
+    int occurence = keySeen[key];
+    if (occurence != 0) {
+      REprintf("Encounter a duplicated site: [ %s ] in file [ %s ]\n", key.c_str(), fn.c_str());
+      key += '/';
+      key += toStr(occurence);
+    }
+    keySeen[key]++;
+
+    if ((*location)[gene].count(key) != 0) {  // should not happen
+      // REprintf("Probably an eror is happening... key = [ %s ]\n",
+      // key.c_str());
       continue;
+    }
     val = (*location)[gene].size();
     (*location)[gene][key] = val;
   }
@@ -233,7 +264,7 @@ SEXP impl_rvMetaReadData(
 
   if (FLAG_pvalFile.size() != FLAG_covFile.size()) {
     if (FLAG_covFile.size() == 0) {
-      Rprintf("Skip loading covaraince file!\n");
+      Rprintf("Skip loading covariance file!\n");
     } else {
       Rprintf("Unequal size between score file and cov file!\n");
       Rprintf("Quitting...");
@@ -265,7 +296,9 @@ SEXP impl_rvMetaReadData(
     for (int i = 0; i < nStudy; ++i) {
       sortLocationPerGene(&(geneLocationMap[key]));
     }
-  };
+  }
+  // annotation text consumes lots of memor. This stores position=>annotation
+  // mapping
   std::map<std::string, std::set<std::string> > posAnnotationMap;
 
   //////////////////////////////////////////////////
@@ -340,9 +373,9 @@ SEXP impl_rvMetaReadData(
     SEXP s = VECTOR_ELT(ret, i);
     numAllocated +=
         createList(names.size(), &s);  // a list with 10 or more elements:
-                                       // ref, alt, n, maf, stat,
-                                       // direction, p, cov, pos,
-                                       // anno, ...
+    // ref, alt, n, maf, stat,
+    // direction, p, cov, pos,
+    // anno, ...
     numAllocated += setListNames(names, &s);
 
     SEXP ref, alt, n, af, ac, callRate, hwe, nref, nhet, nalt, ustat, vstat,
@@ -450,7 +483,7 @@ SEXP impl_rvMetaReadData(
 
       // Rprintf("Create double array %d for study %d\n", npos * npos, j);
       if (FLAG_covFile.empty()) {
-        /// if skip covFile, then just set cov to be 1 by 1 matrix of NA
+        /// if skip covFile, just set cov to be 1 by 1 matrix of NA
         numAllocated += createDoubleArray(1, &t);
         numAllocated += setDim(1, 1, &t);
       } else {
@@ -610,9 +643,16 @@ SEXP impl_rvMetaReadData(
     SET_VECTOR_ELT(s, RET_N_CTRL_INDEX, nCtrl);
 
     SET_VECTOR_ELT(ret, i, s);
-  };
+  }
   // REprintf("finish allocate memory\n");
 
+  // cov file does not have ref/alt allele,
+  // we will record how to map variant location to its index
+  std::vector<std::vector<std::map<std::string, int> > > covLocation2indice(geneRange.size());
+  for (size_t i = 0; i != geneRange.size(); ++i) {
+    covLocation2indice[i].resize(nStudy);
+  }
+
   // return results
   Rprintf("Read score tests...\n");
   // read pval file and fill in values
@@ -677,7 +717,7 @@ SEXP impl_rvMetaReadData(
       continue;
     }
 
-    // loop per gene
+    // loop per gene for a given study
     for (size_t idx = 0; idx < geneRange.size(); ++idx) {
       const std::string& gene = geneRange.keyAt(idx);
       const std::string& range = geneRange.valueAt(idx);
@@ -686,28 +726,43 @@ SEXP impl_rvMetaReadData(
       if (!geneLocationMap.find(gene)) continue;
       const std::map<std::string, int>& location2idx = geneLocationMap[gene];
 
-      std::set<std::string> processedSite;
+      std::map<std::string, int>& covLocation2idx = covLocation2indice[idx][study];
+      std::set<std::string> processedVariant;
+      std::map<std::string, int> processedLocation;
       TabixReader tr(FLAG_pvalFile[study]);
       tr.addRange(range);
+      tr.mergeRange();
       std::string line;
       std::vector<std::string> fd;
       // temp values
-      std::string p;  // meaning position
+      std::string p;   // meaning position, e.g. 1:100_A/T
+      std::string p2;  // short position, e.g. 1:100, that is for cov file
+
       int tempInt;
       double tempDouble;
       while (tr.readLine(&line)) {
         stringNaturalTokenize(line, " \t", &fd);
         if ((int)fd.size() <= PVAL_FILE_MIN_COLUMN_NUM) continue;
-        p = fd[PVAL_FILE_CHROM_COL];
-        p += ':';
-        p += fd[PVAL_FILE_POS_COL];
-        if (processedSite.count(p) == 0) {
-          processedSite.insert(p);
-        } else {
-          Rprintf("Position %s appeared more than once, skipping...\n",
-                  p.c_str());
-          continue;
+        p = fd[PVAL_FILE_CHROM_COL] + ":" + fd[PVAL_FILE_POS_COL] + "_" +
+            fd[PVAL_FILE_REF_COL] + "/" + fd[PVAL_FILE_ALT_COL];
+        p2 = fd[PVAL_FILE_CHROM_COL] + ":" + fd[PVAL_FILE_POS_COL];
+        if (processedVariant.count(p)) {
+          REprintf(
+              "Error: encounter a duplicated site: [ %s ] in file [ %s ], will overwrite "
+              "previous results!\n",
+              p.c_str(),
+              FLAG_pvalFile[study].c_str());
         }
+        processedVariant.insert(p);
+        int& occurence = processedLocation[p2];
+        if (occurence > 0) {
+          p2 += "/";
+          p2 += toStr(occurence);
+          if (occurence > 1) {
+            REprintf("for debug: is this site has >= 4 alleles??\n");
+          }
+        }
+        occurence++;
 
         SEXP u, v, s;
         // std::string& gene = locationGeneMap[p];
@@ -717,6 +772,10 @@ SEXP impl_rvMetaReadData(
         //     existing position
         // int idx = geneLocationMap[gene][p];
         int idx = location2idx.find(p)->second;
+        covLocation2idx[p2] = idx;
+        // if (covLocation2idx.size() > location2idx.size()) {
+        //   REprintf("%s:%d This should not happen\n", __FILE__, __LINE__);
+        // }
 
         // Rprintf("working on index %d, with position %s\n", idx, p.c_str());
         u = VECTOR_ELT(ret, geneIndex[gene]);
@@ -890,7 +949,7 @@ SEXP impl_rvMetaReadData(
           v = VECTOR_ELT(u, RET_PVAL_INDEX);
           s = VECTOR_ELT(v, study);  // pval
           REAL(s)[idx] = tempDouble;
-        };
+        }
 
         if (PVAL_FILE_ANNO_COL >= 0 && (int)fd.size() >= PVAL_FILE_ANNO_COL) {
           const std::string& s = fd[PVAL_FILE_ANNO_COL];
@@ -924,7 +983,7 @@ SEXP impl_rvMetaReadData(
         std::string ret;
         set2string(posAnnotationMap[it->first], &ret, ',');
         SET_STRING_ELT(anno, idx, mkChar(ret.c_str()));
-      };
+      }
     }
   }
 
@@ -940,6 +999,7 @@ SEXP impl_rvMetaReadData(
     std::vector<std::string> covZZ;
 
     for (int study = 0; study < nStudy; ++study) {
+      Rprintf("In study %d\n", study);
       // parse header
       CovFileFormat covHeader;
       if (covHeader.open(FLAG_covFile[study]) < 0) {
@@ -979,9 +1039,14 @@ SEXP impl_rvMetaReadData(
 
         // if (geneLocationMap.find(gene) == geneLocationMap.end()) continue;
         if (!geneLocationMap.find(gene)) continue;
-        const std::map<std::string, int>& location2idx = geneLocationMap[gene];
+        const std::map<std::string, int>& variant2idx = geneLocationMap[gene];
+        const std::map<std::string, int>& location2idx = covLocation2indice[idx][study];
 
-        std::set<std::string> processedSite;
+        std::map<std::string, int>
+            processedSite;  // across rows, number of analyzed first position
+        std::map<std::string, int> processedSitePerLine;  // within the same
+                                                          // row, number of
+                                                          // position processed
 
         // REprintf("Read file %s ", FLAG_covFile[study].c_str());
         TabixReader tr(FLAG_covFile[study]);
@@ -1006,7 +1071,7 @@ SEXP impl_rvMetaReadData(
           if ((int)fd.size() <= COV_FILE_MIN_COLUMN_NUM) continue;
 
           const std::string& chrom = fd[COV_FILE_CHROM_COL];
-          // REprintf("pos: %s\n", fd[COV_FILE_COV_COL].c_str());
+          // REprintf("cov: %s\n", fd[COV_FILE_COV_COL].c_str());
           stringNaturalTokenize(fd[COV_FILE_POS_COL], ',', &pos);
           stringNaturalTokenize(fd[COV_FILE_COV_COL], ':', &covArray);
           stringNaturalTokenize(covArray[0], ',', &cov);
@@ -1024,7 +1089,7 @@ SEXP impl_rvMetaReadData(
             continue;
           }
 
-          int covLen = location2idx.size();
+          int covLen = variant2idx.size();
           SEXP u, v, s;
           u = VECTOR_ELT(ret, geneIndex[gene]);
           v = VECTOR_ELT(u,
@@ -1032,28 +1097,51 @@ SEXP impl_rvMetaReadData(
           s = VECTOR_ELT(v, study);
 
           std::string pi = chrom + ":" + pos[0];
+          int& occurence_i = processedSite[pi];
+          processedSitePerLine.clear();
+          processedSitePerLine[pi] = occurence_i;
+          if (occurence_i > 0) {  // multi-allelic site
+            pi += '/';
+            pi += toStr(occurence_i);
+          }
           if (location2idx.count(pi) == 0) {
+            REprintf(
+                "Warning: location [ %s ] found in cov file [ %s ] but not "
+                "score file [ %s ]\n",
+                pi.c_str(), FLAG_covFile[study].c_str(),
+                FLAG_pvalFile[study].c_str());
             continue;
           }
           int posi = location2idx.find(pi)->second;
+          occurence_i++;
+
           // REprintf("%s:%d Pos %s = %d, covLen = %d\n", __FILE__, __LINE__,
-          // pi.c_str(), posi, covLen);
+          //          pi.c_str(), posi, covLen);
           std::string pj;
           double tmp;
 
           for (size_t j = 0; j < pos.size(); ++j) {
             pj = chrom + ":" + pos[j];
+            int& occurence_j = processedSitePerLine[pj];
+            if (occurence_j > 0) {  // multi-allelic site
+              pj += '/';
+              pj += toStr(occurence_j);
+            }
+            occurence_j++;
             if (location2idx.count(pj) == 0) {
               continue;
             }
             int posj = location2idx.find(pj)->second;
+
             // REprintf("i = %d, j = %d\n", posi, posj);
+            // if (posi >= covLen || posj >= covLen) {
+            //   REprintf("Something unusal happens at %s:%d\n", __FILE__, __LINE__);
+            // }
             if (str2double(cov[j], &tmp)) {
               REAL(s)[posi * covLen + posj] = tmp;
               REAL(s)[posj * covLen + posi] = tmp;
             }
           }
-
           // check if we have covXZ and covZZ fields
           if (covArray.size() == 1) continue;
 
@@ -1215,9 +1303,18 @@ SEXP impl_readCovByRange(SEXP arg_covFile, SEXP arg_range) {
   std::string line;
   std::vector<std::string> fd;
   std::string chrom;
+  // to accomodate multi-allelic, position can be: 1:100, 1:100/1, 1:100/2 ...
   std::map<std::string, int> pos2idx;
-  std::vector<int> positionPerRow;
-  std::vector<std::string> position;
+  // across rows, number of times the first position has been processed
+  std::map<std::string, int> firstPositionProcessed;
+  // within the same row, number of times position has been processed
+  // must have this to support adjacent multi-allelic sites
+  // e.g.
+  // line 1: pos = 100,100,200,200
+  // line 2: pos = 100,200,200
+  std::map<std::string, int> positionProcessedInRow;
+  std::vector<int> positionPerRow;    // number of pos per row in the cov file
+  std::vector<std::string> position;  // record positions - used a dim names
 
   // these are used each line
   std::vector<std::string> cov;
@@ -1269,15 +1366,31 @@ SEXP impl_readCovByRange(SEXP arg_covFile, SEXP arg_range) {
       }
 
       int considerPos = 0;
+      positionProcessedInRow.clear();
       for (size_t i = 0; i < fdPos.size(); ++i) {
         // Rprintf("fdPos[%zu] = %s\n", i, fdPos[i].c_str());
         int p = atoi(fdPos[i]);
         if (p > end) continue;
         // consider in-range positions only (tabix already consider p >= beg)
+
+        if (i == 0) {
+          positionProcessedInRow[fdPos[i]] = firstPositionProcessed[fdPos[i]];
+          firstPositionProcessed[fdPos[i]]++;
+        }
+
+        // add suffix to fdPos[i] if necessary
+        int& occurence = positionProcessedInRow[fdPos[i]];
+        if (occurence > 0) {
+          fdPos[i] += '/';
+          fdPos[i] += toStr(occurence);
+        }
+
         if (pos2idx.count(fdPos[i]) == 0) {
           pos2idx[fdPos[i]] = position.size();
           position.push_back(fdPos[i]);
         }
+
+        ++occurence;
         ++considerPos;
       }
       // REprintf("fdPos.size() = %zu, considerPos = %d\n", fdPos.size(),
@@ -1305,7 +1418,7 @@ SEXP impl_readCovByRange(SEXP arg_covFile, SEXP arg_range) {
       }
       positionPerRow.push_back(considerPos);
       // REprintf("considerPos = %d\n", considerPos);
-    }
+    }  // end while
     ti_iter_destroy(iter);
     // Rprintf("parse end\n");
   } else {
@@ -1544,13 +1657,13 @@ SEXP impl_readScoreByRange(SEXP arg_scoreFile, SEXP arg_range) {
     } else {
       annoFull.push_back("");
     }
-  };
+  }
 
   if (fieldLen < 0) {
     REprintf("No valid input line read, please check your file [ %s ]\n",
              FLAG_scoreFile.c_str());
     return ret;
-  };
+  }
 
   // REprintf("Construct return values\n");
   int retListLen;
@@ -1656,7 +1769,7 @@ SEXP impl_readSkewByRange(SEXP arg_skewFile, SEXP arg_range) {
 
   // Note: no need to check these, as rareMetalWorker does not have them:
   // SKEW_FILE_END_POS_COL < 0 || SKEW_FILE_NUM_MARKER_COL < 0 ||
-  
+
   if (SKEW_FILE_CHROM_COL < 0 || SKEW_FILE_START_POS_COL < 0 ||
       SKEW_FILE_MARKER_POS_COL < 0 || SKEW_FILE_SKEW_COL < 0) {
     REprintf("File [ %s ] does not have all necessary headers\n",
